@@ -911,55 +911,61 @@ test_that("Test the vignette", {
   }
 
   set.seed(4711)
-  expect_warning(looss_1 <- loo_subsample(llfun_logistic, draws = parameter_draws, data = stan_df, observations = 100))
+  expect_no_warning(looss_1 <- loo_subsample(llfun_logistic, draws = parameter_draws, data = stan_df, observations = 100))
   expect_output(print(looss_1), "Computed from 4000 by 100 subsampled log-likelihood")
   expect_output(print(looss_1), "values from 3020 total observations.")
+  expect_output(print(looss_1), "MCSE and ESS estimates assume independent draws")
   expect_output(print(looss_1), "elpd_loo  -1968.5 15.6            0.3")
   expect_output(print(looss_1), "p_loo         3.1  0.1            0.4")
   expect_s3_class(looss_1, c("psis_loo_ss", "psis_loo", "loo"))
 
   set.seed(4711)
-  expect_warning(looss_1b <- update(looss_1, draws = parameter_draws, data = stan_df, observations = 200))
+  expect_no_warning(looss_1b <- update(looss_1, draws = parameter_draws, data = stan_df, observations = 200))
   expect_output(print(looss_1b), "Computed from 4000 by 200 subsampled log-likelihood")
   expect_output(print(looss_1b), "values from 3020 total observations.")
+  expect_output(print(looss_1b), "MCSE and ESS estimates assume independent draws")
   expect_output(print(looss_1b), "elpd_loo  -1968.3 15.6            0.2")
   expect_output(print(looss_1b), "p_loo         3.2  0.1            0.4")
   expect_s3_class(looss_1b, c("psis_loo_ss", "psis_loo", "loo"))
 
   set.seed(4711)
-  expect_warning(looss_2 <- loo_subsample(x = llfun_logistic, draws = parameter_draws, data = stan_df, observations = 100, estimator = "hh_pps", loo_approximation = "lpd", loo_approximation_draws = 100))
+  expect_no_warning(looss_2 <- loo_subsample(x = llfun_logistic, draws = parameter_draws, data = stan_df, observations = 100, estimator = "hh_pps", loo_approximation = "lpd", loo_approximation_draws = 100))
   expect_output(print(looss_2), "Computed from 4000 by 100 subsampled log-likelihood")
   expect_output(print(looss_2), "values from 3020 total observations.")
+  expect_output(print(looss_2), "MCSE and ESS estimates assume independent draws")
   # Currently failing
   # expect_output(print(looss_2), "elpd_loo  -1968.9 15.4            0.5")
   # expect_output(print(looss_2), "p_loo         3.5  0.2            0.5")
   expect_s3_class(looss_2, c("psis_loo_ss", "psis_loo", "loo"))
 
   set.seed(4711)
-  expect_warning(aploo_1 <- loo_approximate_posterior(llfun_logistic, draws = parameter_draws_laplace, data = stan_df, log_p = log_p, log_g = log_g))
+  expect_no_warning(aploo_1 <- loo_approximate_posterior(llfun_logistic, draws = parameter_draws_laplace, data = stan_df, log_p = log_p, log_g = log_g))
   expect_output(print(aploo_1), "Computed from 2000 by 3020 log-likelihood matrix")
+  expect_output(print(aploo_1), "MCSE and ESS estimates assume independent draws")
   expect_output(print(aploo_1), "elpd_loo  -1968.4 15.6")
   expect_output(print(aploo_1), "p_loo         3.2  0.2")
   expect_output(print(aploo_1), "Posterior approximation correction used.")
-  expect_output(print(aploo_1), "\\(-Inf, 0.5\\]   \\(good\\)     2989  99.0")
-  expect_output(print(aploo_1), "\\(0.5, 0.7\\]   \\(ok\\)         31   1.0")
+  expect_output(print(aploo_1), "All Pareto k estimates are good")
+  expect_equal(length(pareto_k_ids(aploo_1,threshold=0.5)), 31)
   expect_s3_class(aploo_1, c("psis_loo_ap", "psis_loo", "loo"))
 
   set.seed(4711)
-  expect_warning(looapss_1 <- loo_subsample(llfun_logistic, draws = parameter_draws_laplace, data = stan_df, log_p = log_p, log_g = log_g, observations = 100))
+  expect_no_warning(looapss_1 <- loo_subsample(llfun_logistic, draws = parameter_draws_laplace, data = stan_df, log_p = log_p, log_g = log_g, observations = 100))
   expect_output(print(looapss_1), "Computed from 2000 by 100 subsampled log-likelihood")
+  expect_output(print(looapss_1), "MCSE and ESS estimates assume independent draws")
   expect_output(print(looapss_1), "values from 3020 total observations.")
   expect_output(print(looapss_1), "elpd_loo  -1968.2 15.6            0.4")
   expect_output(print(looapss_1), "p_loo         2.9  0.1            0.5")
-  expect_output(print(looapss_1), "\\(-Inf, 0.5\\]   \\(good\\)     97    97.0")
-  expect_output(print(looapss_1), "\\(0.5, 0.7\\]   \\(ok\\)        3     3.0")
+  expect_output(print(looapss_1), "All Pareto k estimates are good")
+  expect_equal(length(pareto_k_ids(looapss_1,threshold=0.5)), 3)
 
   # Loo compare
   set.seed(4711)
-  expect_warning(looss_1 <- loo_subsample(llfun_logistic, draws = parameter_draws, data = stan_df, observations = 100))
+  expect_no_warning(looss_1 <- loo_subsample(llfun_logistic, draws = parameter_draws, data = stan_df, observations = 100))
   set.seed(4712)
-  expect_warning(looss_2 <- loo_subsample(x = llfun_logistic, draws = parameter_draws_2, data = stan_df2, observations = 100))
+  expect_no_warning(looss_2 <- loo_subsample(x = llfun_logistic, draws = parameter_draws_2, data = stan_df2, observations = 100))
   expect_output(print(looss_2), "Computed from 4000 by 100 subsampled log-likelihood")
+  expect_output(print(looss_2), "MCSE and ESS estimates assume independent draws")
   expect_output(print(looss_2), "values from 3020 total observations.")
   expect_output(print(looss_2), "elpd_loo  -1952.0 16.2            0.2")
   expect_output(print(looss_2), "p_loo         2.6  0.1            0.3")
@@ -968,7 +974,7 @@ test_that("Test the vignette", {
   expect_output(print(comp), "model1 16.5      22.5     0.4")
 
   set.seed(4712)
-  expect_warning(looss_2_m <- loo_subsample(x = llfun_logistic, draws = parameter_draws_2, data = stan_df2, observations = looss_1))
+  expect_no_warning(looss_2_m <- loo_subsample(x = llfun_logistic, draws = parameter_draws_2, data = stan_df2, observations = looss_1))
   expect_message(looss_2_m <- suppressWarnings(loo_subsample(x = llfun_logistic, draws = parameter_draws_2, data = stan_df2, observations = obs_idx(looss_1))),
                  "Simple random sampling with replacement assumed.")
 
@@ -976,12 +982,12 @@ test_that("Test the vignette", {
   expect_output(print(comp), "model1 16.1       4.4     0.1")
 
   set.seed(4712)
-  expect_warning(looss_1 <- update(looss_1, draws = parameter_draws, data = stan_df, observations = 200))
-  expect_warning(looss_2_m <- update(looss_2_m, draws = parameter_draws_2, data = stan_df2, observations = looss_1))
+  expect_no_warning(looss_1 <- update(looss_1, draws = parameter_draws, data = stan_df, observations = 200))
+  expect_no_warning(looss_2_m <- update(looss_2_m, draws = parameter_draws_2, data = stan_df2, observations = looss_1))
   expect_silent(comp2 <- loo_compare(looss_1, looss_2_m))
   expect_output(print(comp2), "model1 16.3       4.4     0.1")
 
-  expect_warning(looss_2_full <- loo(x = llfun_logistic, draws = parameter_draws_2, data = stan_df2))
+  expect_no_warning(looss_2_full <- loo(x = llfun_logistic, draws = parameter_draws_2, data = stan_df2))
   expect_message(comp3 <- loo_compare(x = list(looss_1, looss_2_full)),
                  "Estimated elpd_diff using observations included in loo calculations for all models.")
   expect_output(print(comp3), "model1 16.5       4.4     0.3")
